@@ -49,7 +49,6 @@
 #include <js/GCVector.h>            // for RootedVector
 #include <js/GlobalObject.h>        // for CurrentGlobalOrNull
 #include <js/HashTable.h>           // for DefaultHasher via WeakCache
-#include <js/HeapAPI.h>             // for ExposeObjectToActiveJS
 #include <js/Id.h>
 #include <js/Modules.h>
 #include <js/Promise.h>             // for JobQueue::SavedJobQueue
@@ -68,6 +67,7 @@
 #include <js/friend/DumpFunctions.h>
 #include <jsapi.h>        // for JS_GetFunctionObject, JS_Ge...
 #include <jsfriendapi.h>  // for ScriptEnvironmentPreparer
+#include <mozilla/UniquePtr.h>  // for UniquePtr::get
 
 #include "gi/closure.h"  // for Closure::Ptr, Closure
 #include "gi/function.h"
@@ -362,13 +362,12 @@ void GjsContextPrivate::trace(JSTracer* trc, void* data) {
     JS::TraceEdge<JSObject*>(trc, &gjs->m_main_loop_hook, "GJS main loop hook");
     gjs->m_atoms->trace(trc);
     gjs->m_job_queue.trace(trc);
-    gjs->m_cleanup_tasks.trace(trc);
     gjs->m_object_init_list.trace(trc);
 }
 
 void GjsContextPrivate::warn_about_unhandled_promise_rejections(void) {
     for (auto& kv : m_unhandled_rejection_stacks) {
-        const char *stack = kv.second;
+        const char* stack = kv.second.get();
         g_warning("Unhandled promise rejection. To suppress this warning, add "
                   "an error handler to your promise chain with .catch() or a "
                   "try-catch block around your await expression. %s%s",
@@ -896,6 +895,11 @@ void GjsContextPrivate::schedule_gc_internal(bool force_gc) {
     m_auto_gc_id = g_timeout_add_seconds_full(G_PRIORITY_LOW, 10,
                                               trigger_gc_if_needed, this,
                                               nullptr);
+
+    if (force_gc)
+        g_source_set_name_by_id(m_auto_gc_id, "[gjs] Garbage Collection (Big Hammer)");
+    else
+        g_source_set_name_by_id(m_auto_gc_id, "[gjs] Garbage Collection");
 }
 
 /*
@@ -1106,14 +1110,6 @@ bool GjsContextPrivate::run_jobs_fallible() {
     JS::HandleValueArray args(JS::HandleValueArray::empty());
     JS::RootedValue rval(m_cx);
 
-    if (m_job_queue.length() == 0) {
-        // Check FinalizationRegistry cleanup tasks at least once if there are
-        // no microtasks queued. This may enqueue more microtasks, which will be
-        // appended to m_job_queue.
-        if (!run_finalization_registry_cleanup())
-            retval = false;
-    }
-
     /* Execute jobs in a loop until we've reached the end of the queue.
      * Since executing a job can trigger enqueueing of additional jobs,
      * it's crucial to recheck the queue length during each iteration. */
@@ -1157,11 +1153,6 @@ bool GjsContextPrivate::run_jobs_fallible() {
             }
         }
         gjs_debug(GJS_DEBUG_MAINLOOP, "Completed job %zu", ix);
-
-        // Run FinalizationRegistry cleanup tasks after each job. Cleanup tasks
-        // may enqueue more microtasks, which will be appended to m_job_queue.
-        if (!run_finalization_registry_cleanup())
-            retval = false;
     }
 
     m_draining_job_queue = false;
@@ -1171,44 +1162,6 @@ bool GjsContextPrivate::run_jobs_fallible() {
     return retval;
 }
 
-bool GjsContextPrivate::run_finalization_registry_cleanup() {
-    bool retval = true;
-
-    JS::Rooted<FunctionVector> tasks{m_cx};
-    std::swap(tasks.get(), m_cleanup_tasks);
-    g_assert(m_cleanup_tasks.empty());
-
-    JS::RootedFunction task{m_cx};
-    JS::RootedValue unused_rval{m_cx};
-    for (JSFunction* func : tasks) {
-        gjs_debug(GJS_DEBUG_MAINLOOP,
-                  "Running FinalizationRegistry cleanup callback");
-
-        task.set(func);
-        JS::ExposeObjectToActiveJS(JS_GetFunctionObject(func));
-
-        JSAutoRealm ar{m_cx, JS_GetFunctionObject(func)};
-        if (!JS_CallFunction(m_cx, nullptr, task, JS::HandleValueArray::empty(),
-                             &unused_rval)) {
-            // Same logic as above
-            if (!JS_IsExceptionPending(m_cx)) {
-                if (!should_exit(nullptr))
-                    g_critical(
-                        "FinalizationRegistry callback terminated with "
-                        "uncatchable exception");
-                retval = false;
-                continue;
-            }
-            gjs_log_exception_uncaught(m_cx);
-        }
-
-        gjs_debug(GJS_DEBUG_MAINLOOP,
-                  "Completed FinalizationRegistry cleanup callback");
-    }
-
-    return retval;
-}
-
 class GjsContextPrivate::SavedQueue : public JS::JobQueue::SavedJobQueue {
  private:
     GjsContextPrivate* m_gjs;
@@ -1248,7 +1201,7 @@ js::UniquePtr<JS::JobQueue::SavedJobQueue> GjsContextPrivate::saveJobQueue(
 }
 
 void GjsContextPrivate::register_unhandled_promise_rejection(
-    uint64_t id, GjsAutoChar&& stack) {
+    uint64_t id, JS::UniqueChars&& stack) {
     m_unhandled_rejection_stacks[id] = std::move(stack);
 }
 
@@ -1263,11 +1216,6 @@ void GjsContextPrivate::unregister_unhandled_promise_rejection(uint64_t id) {
     }
 }
 
-bool GjsContextPrivate::queue_finalization_registry_cleanup(
-    JSFunction* cleanup_task) {
-    return m_cleanup_tasks.append(cleanup_task);
-}
-
 void GjsContextPrivate::async_closure_enqueue_for_gc(Gjs::Closure* trampoline) {
     //  Because we can't free the mmap'd data for a callback
     //  while it's in use, this list keeps track of ones that
