@@ -5,15 +5,22 @@
 #ifndef GJS_DEPRECATION_H_
 #define GJS_DEPRECATION_H_
 
+#include <vector>
+
 struct JSContext;
 
 enum GjsDeprecationMessageId {
     None,
     ByteArrayInstanceToString,
     DeprecatedGObjectProperty,
+    ModuleExportedLetOrConst,
 };
 
 void _gjs_warn_deprecated_once_per_callsite(JSContext* cx,
                                             GjsDeprecationMessageId message);
 
+void _gjs_warn_deprecated_once_per_callsite(
+    JSContext* cx, GjsDeprecationMessageId id,
+    const std::vector<const char*>& args);
+
 #endif  // GJS_DEPRECATION_H_
